local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")
local LCo = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-Config")
local ACD = LibStub("AceConfigDialog-3.0")
local ACR = LibStub("AceConfigRegistry-3.0")

local CreateMacro, EditMacro, PickupMacro, GetMacroIndexByName = CreateMacro, EditMacro, PickupMacro, GetMacroIndexByName
local pairs, type, sub, tonumber, format = pairs, type, string.sub, tonumber, string.format

local moneySeparators = {
	["dot"] = LCo["Dot"],
	["symbol"] = LCo["Symbol"],
	["texture"] = LCo["Texture"],
}
local raidModeValues = {
	["enabled"] = LCo["Enabled"],
	["inRaid"] = LCo["Disabled once entering a raid"],
	["inRaidInstance"] = LCo["Disabled once entering a raid instance (will work in pvp and outdoor raids)"],
}
local raidModeTrackerValues = {
	["enabled"] = LCo["Enabled"],
	["inRaid"] = LCo["Disabled once entering a raid"],
	["inRaidInstance"] = LCo["Disabled once entering a raid instance (will work in pvp and outdoor raids)"],
	["hiddenInRaid"] = LCo["Hidden once entering a raid"],
	["hiddenInRaidInstance"] = LCo["Hidden once entering a raid instance (will work in pvp and outdoor raids)"],
}
local zSorting = {
	["levelA"] = LCo["Sort zones based on level (ascending)"],
	["levelD"] = LCo["Sort zones based on level (descending)"],
	["nameA"] = LCo["Sort zones based on zone name (ascending)"],
	["nameD"] = LCo["Sort zones based on zone name (descending)"],
}
local zwlPosition = {
	[0] = LCo["Zones without level first"],
	[100] = LCo["Zones without level last"]
}
local qSorting = {
	["weightedA"] = LCo["Sort quests based on weighted level (ascending)"],
	["weightedD"] = LCo["Sort quests based on weighted level (descending)"],
	["levelA"] = LCo["Sort quests based on quest level (ascending)"],
	["levelD"] = LCo["Sort quests based on quest level (descending)"],
	["nameA"] = LCo["Sort quests based on name (ascending)"],
	["nameD"] = LCo["Sort quests based on name (descending)"],
}

local optMGetter, optMSetter, optRMGetter, optRMSetter, optQGetter, optQSetter, partyHeaderSetter, optQMSetter, hexColorGetter, hexColorSetter, optWQLSetter, optCCFSetter, optTagSetter, sortSetter, optHideBTr, specialItemMacro, hideCustomColorMode, zeroColorGetter, zeroColorSetter, halfColorGetter, halfColorSetter, fullColorGetter, fullColorSetter, setDefaultColor, setCustomColorMode, getCustomColorMode, hideRaidMode
do
	function optMGetter(info)
		return core:getModuleEnabled(info[#info])
	end
	function optMSetter(info, value)
		core:setModuleEnabled(info[#info], value)
		if info[#info] == "questsTracker" then
			if not value then
				core.cbh:Fire("questsTrackerDisabled")
			else
				core.cbh:Fire("questsTrackerEnabled")
			end
		end
	end
	function optRMGetter(info)
		return core.db.profile.raidModules[info[#info]]
	end
	function optRMSetter(info, value)
		local oldState = core.db.profile.raidModules.enabled
		core.db.profile.raidModules[info[#info]] = value
		core:updateRaidMode(oldState)
	end
	function optQGetter(info)
		return core.db.profile.quests[info[#info]]
	end
	function optQSetter(info, value)
		core.db.profile.quests[info[#info]] = value
	end
	function partyHeaderSetter(info, value)
		core.db.profile.quests[info[#info]] = value
		if value then
			core:setPartyHeader()
		else
			core:removePartyHeader()
		end
	end
	function optQMSetter(info, value)
		core.db.profile.quests[info[#info]] = value
		core.cbh:Fire("updateAchievementTracker")
	end
	function hexColorGetter(info)
		return tonumber(sub(core.db.profile.quests[info[#info]], 1, 2), 16) / 255, tonumber(sub(core.db.profile.quests[info[#info]], 3, 4), 16) / 255, tonumber(sub(core.db.profile.quests[info[#info]], 5, 6), 16) / 255
	end
	function hexColorSetter(info, arg1, arg2, arg3)
		core.db.profile.quests[info[#info]] = format("%02x%02x%02x", arg1*255, arg2*255, arg3*255)
		core:setQuestStatusString()
		core:computeQuestsStatus()
	end
	function optWQLSetter(info, value)
		core.db.profile.quests[info[#info]] = value
		core:computeQuestLevel()
	end
	function optCCFSetter(info, value)
		core.db.profile.quests[info[#info]] = value
		core:computeQuestsObjectivesStatus()
	end
	function optTagSetter(info, value)
		core.db.profile.quests[info[#info]] = value
		core:computeQuestTag()
	end
	function sortSetter(info, value)
		core.db.profile.quests[info[#info]] = value
		core:allSort()
	end
	function optHideBTr(info, value)
		core.db.profile.quests[info[#info]] = value
		core:hideShowDefaultBlizTracker()
	end
	function specialItemMacro(info, value)
		local index = GetMacroIndexByName("tomQuest2_SI")
		if index == 0 then
			index = CreateMacro("tomQuest2_SI", 1, "", 0)
			PickupMacro(index)
		else
			EditMacro(index, "tomQuest2_SI", 1, "")
		end
	end
	function hideCustomColorMode()
		if core.db.profile.quests.customColorMode then
			return true
		else
			return false
		end
	end
	local function normalizeColorComponent(component)
		if component > 1 then return 1 elseif component < 0 then return 0 else return component end
	end
	function zeroColorGetter(info)
		return normalizeColorComponent(core.db.profile.quests.rj0) , normalizeColorComponent(core.db.profile.quests.gj0), normalizeColorComponent(core.db.profile.quests.bj0)
	end
	function halfColorGetter(info)
		return normalizeColorComponent(0.5*core.db.profile.quests.ri1 + core.db.profile.quests.rj1), normalizeColorComponent(0.5*core.db.profile.quests.gi1 + core.db.profile.quests.gj1), normalizeColorComponent(0.5*core.db.profile.quests.bi1 + core.db.profile.quests.bj1)
	end
	function fullColorGetter(info)
		return normalizeColorComponent(core.db.profile.quests.rj3), normalizeColorComponent(core.db.profile.quests.gj3), normalizeColorComponent(core.db.profile.quests.bj3)
	end
	local function computeFirstFunction(r0, g0, b0, rh, gh, bh)
		core.db.profile.quests.rj1 = r0
		core.db.profile.quests.gj1 = g0
		core.db.profile.quests.bj1 = b0
		core.db.profile.quests.ri1 = 2 * (rh - r0)
		core.db.profile.quests.gi1 = 2 * (gh - g0)
		core.db.profile.quests.bi1 = 2 * (bh - b0)
	end
	local function computeSecondFunction(rh, gh, bh, r1, g1, b1)
		core.db.profile.quests.rj2 = 2 * rh - r1
		core.db.profile.quests.gj2 = 2 * gh - g1
		core.db.profile.quests.bj2 = 2 * bh - b1
		core.db.profile.quests.ri2 = 2 * (r1 - rh)
		core.db.profile.quests.gi2 = 2 * (g1 - gh)
		core.db.profile.quests.bi2 = 2 * (b1 - bh)
	end
	local function zeroColorSetter(info, arg1, arg2, arg3)
		core.db.profile.quests.rj0 = arg1
		core.db.profile.quests.gj0 = arg2
		core.db.profile.quests.bj0 = arg3
		local rh, gh, bh = halfColorGetter()
		computeFirstFunction(arg1, arg2, arg3, rh, gh, bh)
		core:computeQuestsObjectivesStatus()
	end
	function halfColorSetter(info, arg1, arg2, arg3)
		local r0, g0, b0 = zeroColorGetter()
		computeFirstFunction(r0, g0, b0, arg1, arg2, arg3)
		local r1, g1, b1 = fullColorGetter()
		computeSecondFunction(arg1, arg2, arg3, r1, g1, b1)
		core:computeQuestsObjectivesStatus()
	end
	function fullColorSetter(info, arg1, arg2, arg3)
		local rh, gh, bh = halfColorGetter()
		computeSecondFunction(rh, gh, bh, arg1, arg2, arg3)
		core.db.profile.quests.rj3 = arg1
		core.db.profile.quests.gj3 = arg2
		core.db.profile.quests.bj3 = arg3
		core:computeQuestsObjectivesStatus()
	end
	function setDefaultColor()
		core.db.profile.quests.rj0, core.db.profile.quests.gj0, core.db.profile.quests.bj0 = 1, 0, 0
		core.db.profile.quests.ri1, core.db.profile.quests.rj1, core.db.profile.quests.gi1, core.db.profile.quests.gj1, core.db.profile.quests.bi1, core.db.profile.quests.bj1 = 0, 1, 2, 0, 0 ,0
		core.db.profile.quests.ri2, core.db.profile.quests.rj2, core.db.profile.quests.gi2, core.db.profile.quests.gj2, core.db.profile.quests.bi2, core.db.profile.quests.bj2 = -2, 2, 0, 1, 0, 0
		core.db.profile.quests.rj3, core.db.profile.quests.gj3, core.db.profile.quests.bj3 = 0, 1, 0
		core:computeQuestsObjectivesStatus()
	end
	function hideRaidMode()
		if core.db.profile.raidModules.enabled then
			return false
		else
			return true
		end
	end
end

local options, moduleOptions = nil, {}
local function getOptions()
	if not options then
		options = {
			type = "group",
			name = "tomQuest2",
			args = {
				main = {
					type = "group",
					childGroups = "tab",
					name = "main",
					get = optQGetter,
					set = optQSetter,
					args = {
						modules = {
							type = "group",
							name = LCo["Modules"],
							get = optMGetter,
							set = optMSetter,
							order = 1,
							args = {
							},
						},
						raidMode = {
							type = "group",
							name = LCo["Raid mode"],
							get = optRMGetter,
							set = optRMSetter,
							order = 2,
							args = {
								enabled = {
									type = "toggle",
									name = LCo["Raid mode"],
									width = "full",
									order = 1,
								},
								raidModeDesc = {
									type = "description",
									name = LCo["Define module state when in a raid"],
									order = 2,
								},
							}
						},
						other = {
							type = "group",
							name = LCo["Other"],
							order = 3,
							args = {
								partyHeader = {
									name = LCo["Show all party quests you don't have under a specific header"],
									type = "toggle",
									set = partyHeaderSetter,
									order = 1,
									width = "full",
								},
								colorizeMoney = {
									name = LCo["Color money"],
									type= "toggle",
									set = optQMSetter,
									order = 2,
									width = "full",
								},
								moneySeparator = {
									name = LCo["Money separator"],
									type= "select",
									set = optQMSetter,
									values = moneySeparators,
									order = 3,
									width = "full",
								},
								hideDefaultBlizTracker = {
									name = LCo["Hide default blizzard quests and achievement tracker"],
									type= "toggle",
									set = optHideBTr,
									order = 4,
									width = "full",
								},
								macro = {
									name = LCo["Macro Generator"],
									type = "execute",
									func = specialItemMacro,
									order = 5,
								},
						},
						},
						weightedLevel = {
							type = "group",
							name = LCo["Weighted quest level computation"],
							set = optWQLSetter,
							order = 4,
							args = {
								eliteAddLevel = {
									name = LCo["Level to add for an elite quest"],
									type = "range",
									min = 0,
									max = 5,
									step = 1,
									width = "full",
									order = 1,
								},
								groupAddLevel = {
									name = LCo["Level to add per missing player for a group quest"],
									type = "range",
									min = 0,
									max = 5,
									step = 1,
									width = "full",
									order = 2,
								},
								dungeonAddLevel = {
									name = LCo["Level to add per missing player for a dungeon quest"],
									type = "range",
									min = 0,
									max = 5,
									step = 1,
									width = "full",
									order = 3,
								},
								heroicAddLevel = {
									name = LCo["Level to add per missing player for an heroic quest"],
									type = "range",
									min = 0,
									max = 5,
									step = 1,
									width = "full",
									order = 4,
								},
								raidAddLevel = {
									name = LCo["Level to add per missing player for a raid quest"],
									type = "range",
									min = 0,
									max = 5,
									step = 1,
									width = "full",
									order = 5,
								},
							},
						},
						questsTag = {
							type = "group",
							name = LCo["Custom quests tag"],
							set = optTagSetter,
							order = 5,
							args = {
								elite = {
									name = LCo["Elite quest tag"],
									type = "input",
									order = 1,
								},
								group = {
									name = LCo["Group quest tag"],
									type = "input",
									order = 2,
								},
								pvp = {
									name = LCo["PVP quest tag"],
									type = "input",
									order = 3,
								},
								raid = {
									name = LCo["Raid quest tag"],
									type = "input",
									order = 4,
								},
								dungeon = {
									name = LCo["Dungeon quest tag"],
									type = "input",
									order = 5,
								},
								heroic = {
									name = LCo["Heroic quest tag"],
									type = "input",
									order = 6,
								},
								daily = {
									name = LCo["Daily quest tag"],
									type = "input",
									order = 7,
								},
							},
						},
						customColor = {
							type = "group",
							name = LCo["Custom objective coloring function"],
							set = optCCFSetter,
							order = 6,
							args = {
								easyColorSetupDesc = {
									type = "description",
									name = LCo["You can define a color for percentage of completion = 0, 50% and 100%. tomQuest2 will automatically define the needed color function between those threee points"],
									width = "full",
									order = 0,
								},
								zeroColor = {
									name = LCo["Color for completion = 0%"],
									type = "color",
									hasAlpha = false,
									get = zeroColorGetter,
									set = zeroColorSetter,
									width = "full",
									order = 1,
								},
								halfColor = {
									name = LCo["Color for completion = 50%"],
									type = "color",
									hasAlpha = false,
									get = halfColorGetter,
									set = halfColorSetter,
									width = "full",
									order = 2,
								},
								fullColor = {
									name = LCo["Color for completion = 100%"],
									type = "color",
									hasAlpha = false,
									get = fullColorGetter,
									set = fullColorSetter,
									width = "full",
									order = 3,
								},
								resetColor = {
									name = LCo["Reset color functions"],
									type = "execute",
									func = setDefaultColor,
									width = "full",
									order = 4,
								},
								customColorMode = {
									name = LCo["Hide custom color function setup"],
									type = "toggle",
									get = optQGetter,
									set = optQSetter,
									width = "full",
									order = 5,
								},
								customColorFunctionDesc = {
									type = "description",
									name = LCo["Custom objective coloring allows you to define multiplicative and additive component of the coloring function. For each color component you'll have a function that is color = (multiplicative component) * percent + (additive component) where percent is the percent of completion of the objective."],
									width = "full",
									hidden = hideCustomColorMode,
									order = 6,
								},
								zeroCustomColorFunctionHeader = {
									type = "header",
									name = LCo["Function for objective completion at 0%"],
									width = "full",
									hidden = hideCustomColorMode,
									order = 7,
								},
								rj0 = {
									name = LCo["Red add."],
									type = "range",
									min = -5,
									max = 5,
									step = 0.1,
									hidden = hideCustomColorMode,
									order = 8,
								},
								gj0 = {
									name = LCo["Green add."],
									type = "range",
									min = -5,
									max = 5,
									step = 0.1,
									hidden = hideCustomColorMode,
									order = 9,
								},
								bj0 = {
									name = LCo["Blue add."],
									type = "range",
									min = -5,
									max = 5,
									step = 0.1,
									hidden = hideCustomColorMode,
									order = 10
								},
								firstCustomColorFunctionHeader = {
									type = "header",
									name = LCo["Function for [0%, 50%[ objective completion"],
									hidden = hideCustomColorMode,
									order = 11,
								},
								ri1 = {
									name = LCo["Red mult."],
									type = "range",
									min = -5,
									max = 5,
									step = 0.1,
									hidden = hideCustomColorMode,
									order = 12,
								},
								gi1 = {
									name = LCo["Green mult."],
									type = "range",
									min = -5,
									max = 5,
									step = 0.1,
									hidden = hideCustomColorMode,
									order = 13,
								},
								bi1 = {
									name = LCo["Blue mult."],
									type = "range",
									min = -5,
									max = 5,
									step = 0.1,
									hidden = hideCustomColorMode,
									order = 14,
								},
								rj1 = {
									name = LCo["Red add."],
									type = "range",
									min = -5,
									max = 5,
									step = 0.1,
									hidden = hideCustomColorMode,
									order = 15,
								},
								gj1 = {
									name = LCo["Green add."],
									type = "range",
									min = -5,
									max = 5,
									step = 0.1,
									hidden = hideCustomColorMode,
									order = 16,
								},
								bj1 = {
									name = LCo["Blue add."],
									type = "range",
									min = -5,
									max = 5,
									step = 0.1,
									hidden = hideCustomColorMode,
									order = 17,
								},
								secondCustomColorFunctionHeader = {
									type = "header",
									name = LCo["Function for ]50%, 100%[ objective completion"],
									hidden = hideCustomColorMode,
									order = 18,
								},
								ri2 = {
									name = LCo["Red mult."],
									type = "range",
									min = -5,
									max = 5,
									step = 0.1,
									hidden = hideCustomColorMode,
									order = 19,
								},
								gi2 = {
									name = LCo["Green mult."],
									type = "range",
									min = -5,
									max = 5,
									step = 0.1,
									hidden = hideCustomColorMode,
									order = 20,
								},
								bi2 = {
									name = LCo["Blue mult."],
									type = "range",
									min = -5,
									max = 5,
									step = 0.1,
									hidden = hideCustomColorMode,
									order = 21,
								},
								rj2 = {
									name = LCo["Red add."],
									type = "range",
									min = -5,
									max = 5,
									step = 0.1,
									hidden = hideCustomColorMode,
									order = 22,
								},
								gj2 = {
									name = LCo["Green add."],
									type = "range",
									min = -5,
									max = 5,
									step = 0.1,
									hidden = hideCustomColorMode,
									order = 23,
								},
								bj2 = {
									name = LCo["Blue add."],
									type = "range",
									min = -5,
									max = 5,
									step = 0.1,
									hidden = hideCustomColorMode,
									order = 24,
								},
								fullCustomColorFunctionHeader = {
									type = "header",
									name = LCo["Function for objective completion at 100%"],
									hidden = hideCustomColorMode,
									order = 25,
								},
								rj3 = {
									name = LCo["Red add."],
									type = "range",
									min = -5,
									max = 5,
									step = 0.1,
									hidden = hideCustomColorMode,
									order = 26,
								},
								gj3 = {
									name = LCo["Green add."],
									type = "range",
									min = -5,
									max = 5,
									step = 0.1,
									hidden = hideCustomColorMode,
									order = 27,
								},
								bj3 = {
									name = LCo["Blue add."],
									type = "range",
									min = -5,
									max = 5,
									step = 0.1,
									hidden = hideCustomColorMode,
									order = 28,
								},
							},
						},
						questStatusColor = {
							type = "group",
							name = LCo["Quest status color"],
							get = hexColorGetter,
							set = hexColorSetter,
							order = 7,
							args = {
								completedColor = {
									name = LCo["Color for quest complete"],
									type = "color",
									hasAlpha = false,
									width = "full",
									order = 1,
								},
								failedColor = {
									name = LCo["Color for quest failed"],
									type = "color",
									hasAlpha = false,
									width = "full",
									order = 2,
								},
							},
						},
						sorting = {
							type = "group",
							name = LCo["Zones and Quests sorting"],
							set = sortSetter,
							order = 8,
							args = {
								zoneSorting = {
									type = "select",
									name = LCo["Zones sorting method"],
									values = zSorting,
									width = "full",
									order = 1,
								},
								zoneWithoutLvlPos = {
									type = "select",
									name = LCo["Position of zones without level when sorting zones by level"],
									values = zwlPosition,
									width = "full",
									order = 2,
								},
								questsSorting = {
									type = "select",
									name = LCo["Quests sorting method"],
									values = qSorting,
									width = "full",
									order = 3,
								},
							},
						},
					},
				},
			},
		}
		local moduleIndex = 0
		for k, v in core:IterateModules() do
			moduleIndex = moduleIndex + 1
			options.args.main.args.modules.args[k] = {
				type = "toggle",
				name = v.modName,
				width = "double",
				order = moduleIndex,
			}
		end
		local raidModuleIndex = 0
		for k, v in core:IterateModules() do
			raidModuleIndex = raidModuleIndex + 1
			if k == "achievementTracker" or k == "questsTracker" then
				options.args.main.args.raidMode.args[k] = {
					type = "select",
					values = raidModeTrackerValues,
					name = v.modName,
					width = "full",
					hidden = hideRaidMode,
					order = raidModuleIndex + 2,
				}
			else
				options.args.main.args.raidMode.args[k] = {
					type = "select",
					values = raidModeValues,
					name = v.modName,
					width = "full",
					hidden = hideRaidMode,
					order = raidModuleIndex + 2,
				}
			end

		end
		for k,v in pairs(moduleOptions) do
			options.args[k] = (type(v) == "function") and v() or v
		end
	end
	return options
end

function core:SetupOptions()
	InterfaceOptionsFrame:SetFrameStrata("DIALOG") 
	core.optionsFrames = {}
	ACR:RegisterOptionsTable("tomQuest2", getOptions)
	self.optionsFrames.tomQuest2 = ACD:AddToBlizOptions("tomQuest2", "tomQuest2", nil, "main")
	self:registerModuleOptions("Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db), LCo["Profiles"])
end

function core:registerModuleOptions(name, optionTbl, displayName)
	moduleOptions[name] = optionTbl
	self.optionsFrames[name] = ACD:AddToBlizOptions("tomQuest2", displayName, "tomQuest2", name)
end